Ext.namespace('Zarafa.plugins.owncloudrcvattachment.dialogs');

/**
 * @class Zarafa.plugins.owncloudrcvattachment.dialogs.FolderTreePanel
 * @extends Ext.tree.TreePanel
 * Shows tree of all user files from Owncloud
 */
Zarafa.plugins.owncloudrcvattachment.dialogs.FolderTreePanel = Ext.extend(Ext.tree.TreePanel, {

	selectedFolder : null,
	
	response : null,

	/**
	 * @constructor
	 * @param {Object} config
	 */
	constructor : function(config)
	{
		config = config || {};
		this.response = config.response;
		
		Ext.applyIf(config, {
			root: {
				nodeType: 'async',
				text: '/',
				id: container.getSettingsModel().get('zarafa/v1/contexts/owncloud/owncloud_path') + "/",
				expanded : true
			},
			autoScroll: true,
			viewConfig: {
				style	:	{ overflow: 'auto', overflowX: 'hidden' }
			},
			maskDisabled: true,
			listeners: {
				click: function(n) {
					this.selectedFolder = n.attributes.id.substring(container.getSettingsModel().get('zarafa/v1/contexts/owncloud/owncloud_path').length);
				}
			},
			loader : new Zarafa.owncloud.data.DirectoryLoader({loadfiles: false}),
			buttons : [
				this.createActionButtons()
			]
		});
		Zarafa.plugins.owncloudrcvattachment.dialogs.FolderTreePanel.superclass.constructor.call(this, config);
	},

	/**
	 * Creates action button for adding file
	 * from Owncloud to email as attachment
	 * @return {Array}
	 * @private
	 */
	createActionButtons : function()
	{
		return [
		{
			xtype   : 'button',
			text    : _('New Folder'),
			tooltip : {
				title   : _('New Folder'),
				text    : _('Create a new folder in owncloud')
			},
			width	: 100,
			handler : this.newFolder,
			scope   : this
		},
		{
			xtype   : 'button',
			text    : _('&nbsp;&nbsp;Save'),
			tooltip : {
				title   : _('Store Attachment'),
				text    : _('Store attachment to the selected owncloud folder.')
			},
			width	: 65,
			iconCls : 'icon_owncloud',
			handler : this.uploadFile,
			scope   : this
		}		
		];
	},

	/**
	 * Uploads the prepared attachment to Owncloud
	 * @private
	 */
	uploadFile : function()
	{
		if(this.selectedFolder == null) {
			Ext.MessageBox.show({
				title   : _('Warning'),
				msg     : _('You have to choose a folder!'),
				icon    : Ext.MessageBox.WARNING,
				buttons : Ext.MessageBox.OK
			});
		} else {
			try {
				this.dialog.treePanel.disable();
				container.getRequest().singleRequest(
					'owncloudmodule',
					'uploadtooc',
					{
						tmpfile : this.response.tmpname,
						destdir : this.selectedFolder,
						destname : this.response.filename
					},
					new Zarafa.owncloud.data.ResponseHandler({
						successCallback : this.uploadDone.createDelegate(this)
					})
				);
			} catch (e) {
				Ext.MessageBox.show({
					title   : _('Warning'),
					msg     : e.getMessage(),
					icon    : Ext.MessageBox.WARNING,
					buttons : Ext.MessageBox.OK
				});
			}
		}
	},
	
	/**
	 * Upload done =)
	 * @param {Object} response
	 * @private
	 */
	uploadDone : function(response)
	{
		if(response.status == 201) {
			container.getNotifier().notify('info', 'Uploaded', 'Attachment successfully stored to owncloud!');
		} else {
			container.getNotifier().notify('error', 'Upload Failed', 'Attachment could not be stored in owncloud! Error: ' + response.status);
		}
		this.dialog.close();
	},	
	
	/**
	 * Creates a new owncloud folder
	 * @private
	 */
	newFolder : function()
	{
		if(this.selectedFolder == null) {
			Ext.MessageBox.show({
				title   : _('Warning'),
				msg     : _('You have to choose a folder!'),
				icon    : Ext.MessageBox.WARNING,
				buttons : Ext.MessageBox.OK
			});
		} else {
			Zarafa.common.dialogs.MessageBox.prompt('Folder Name', 'Please enter a foldername', this.createRemoteFolder, this);
		}
	},
	
	createRemoteFolder : function(button, text)
	{
		if(button == "ok") {
			try {
				container.getRequest().singleRequest(
					'owncloudmodule',
					'createdir',
					{
						dirname : this.selectedFolder + text,
						basedir : this.selectedFolder
					},
					new Zarafa.owncloud.data.ResponseHandler({
						successCallback : this.createDirDone.createDelegate(this)
					})
				);
			} catch (e) {
				console.log(e);
				Ext.MessageBox.show({
					title   : _('Warning'),
					msg     : e.getMessage(),
					icon    : Ext.MessageBox.WARNING,
					buttons : Ext.MessageBox.OK
				});
			}
		}
	},
	
	/**
	 * createDirDone =)
	 * @param {Object} response
	 * @private
	 */
	createDirDone : function(response)
	{
		if(response.status == 201) {
			container.getNotifier().notify('info', 'Created', 'Directory created!');
			this.dialog.treePanel.getNodeById(response.basedir).reload();
		} else {
			container.getNotifier().notify('error', 'Creation Failed', 'Directory not created!');
		}
	}
});

Ext.reg('owncloudrcvattachment.folderpanel', Zarafa.plugins.owncloudrcvattachment.dialogs.FolderTreePanel);
